; ****************************************************************************
; ******                                                                ******
; ******                      - JACK! THE RIPPER -                      ******
; ******                 By Alain BROBECKER. (aka baah)                 ******
; ******                    v1.00 - 24 november 1995                    ******
; ******                                                                ******
; ****************************************************************************

            include rincbin.inc
            scr_height=200
            scr_size=320*scr_height


            .386

; ****************************************************************************
; ****************************************************************************
; **********************************  CODE  **********************************
; ****************************************************************************
; ****************************************************************************
THE_CODE    SEGMENT USE16
            Assume CS:THE_CODE,SS:THE_STACK

proggy_start:
  mov       ax,13h                  ; Switch to 320*200 linear mode.
  int       10h
  mov       ax,0a000h               ; Put screen base in es.
  mov       es,ax

  mov       ax,0305h                ; Set keyboard repetition speed.
  mov       bx,0002h
  int       16h

  call      put_palette

init_values:
  mov       word ptr ram_width,320
  mov       word ptr ram_modulo,0
  mov       word ptr ram_total,320

redraw_page:                        ; Here, we redraw the entire screen.
  mov       bx,199
@@copy_one_line:
  call      copy_line
  dec       bx
  jGE       @@copy_one_line

wait_keyboard:
  mov       bx,scr_pointer
  shr       bx,2
  mov       dx,03d4h                ; CRT Controller.
  mov       al,0ch                  ; Start adress high subregister.
  mov       ah,bh
  out       dx,ax
  inc       al                      ; Start adress low subregister.
  mov       ah,bl
  out       dx,ax

  mov       ah,01h                  ; Wait until a char is in keyboard buffer.
  int       16h
  jZ        wait_keyboard
  xor       ah,ah                   ; Read char.
  int       16h

;----> Scroll Up? <-----------------------------------------------------------
  cmp       ax,5000h                ; Cursor down?
  jNE       no_scroll_up
  mov       eax,ram_pointer         ; Modify ram pointer.
  xor       ebx,ebx
  mov       bx,ram_total
  add       eax,ebx
  cmp       eax,640*1024-scr_size
  jG        wait_keyboard
  mov       ram_pointer,eax         ; Save modified pointer.
  add       word ptr scr_pointer,320 ; Modify screen pointer.
  mov       bx,199
  call      copy_line
  jmp       wait_keyboard
no_scroll_up:

;----> Scroll Down? <---------------------------------------------------------
  cmp       ax,4800h                ; Cursor up?
  jNE       no_scroll_down
  mov       eax,ram_pointer         ; Modify ram pointer.
  xor       ebx,ebx
  mov       bx,ram_total
  sub       eax,ebx
  jL        wait_keyboard
  mov       ram_pointer,eax         ; Save modified pointer.
  sub       word ptr scr_pointer,320 ; Modify screen pointer.
  mov       bx,0
  call      copy_line
  jmp       wait_keyboard
no_scroll_down:

;----> Scroll Left? <---------------------------------------------------------
  cmp       ax,4b00h                ; Cursor left?
  jNE       no_scroll_left
  mov       eax,ram_pointer         ; Modify ram pointer.
  dec       eax
  jGE       scroll_left_ok
  xor       eax,eax
scroll_left_ok:
  mov       ram_pointer,eax         ; Save modified pointer.
  jmp       redraw_page
no_scroll_left:

;----> Scroll Right? <--------------------------------------------------------
  cmp       ax,4d00h                ; Cursor right?
  jNE       no_scroll_right
  mov       eax,ram_pointer         ; Modify ram pointer.
  inc       eax
  cmp       eax,640*1024-scr_size
  jLE       scroll_left_ok
  mov       eax,640*1024-scr_size
scroll_right_ok:
  mov       ram_pointer,eax         ; Save modified pointer.
  jmp       redraw_page
no_scroll_right:

  mov       bx,scr_height
;----> One Page Up? <---------------------------------------------------------
  cmp       ax,5100h                ; Page down?
  jNE       no_page_up
  mov       ax,ram_total            ; Modify ram pointer.
  mul       bx                      ; dx:ax=(width+modulo)*scr_height.
  shl       edx,16
  mov       dx,ax                   ; edx=(width+modulo)*scr_height.
  add       edx,ram_pointer
  cmp       edx,640*1024-scr_size
  jLE       page_up_ok
  mov       edx,640*1024-scr_size
page_up_ok:
  mov       ram_pointer,edx         ; Save modified pointer.
  jmp       redraw_page
no_page_up:

;----> One Page Down? <-------------------------------------------------------
  cmp       ax,4900h                ; Page up?
  jNE       no_page_down
  mov       ax,ram_total            ; Modify ram pointer.
  mul       bx                      ; dx:ax=(width+modulo)*scr_height.
  shl       edx,16
  mov       dx,ax                   ; edx=(width+modulo)*scr_height.
  mov       eax,ram_pointer
  sub       eax,edx
  jGE       page_down_ok
  xor       eax,eax
page_down_ok:
  mov       ram_pointer,eax         ; Save modified pointer.
  jmp       redraw_page
no_page_down:

;----> Decrement Width? <-----------------------------------------------------
  cmp       al,31h                  ; '1'?
  jNE       no_dec_width
  mov       ax,ram_width
  dec       ax
  JLE       wait_keyboard
  mov       ram_width,ax
  dec       ram_total
  jmp       redraw_page
no_dec_width:

;----> Increment Width? <-----------------------------------------------------
  cmp       al,32h                  ; '2'?
  jNE       no_inc_width
  mov       ax,ram_width
  inc       ax
  cmp       ax,321
  JGE       wait_keyboard
  mov       ram_width,ax
  inc       ram_total
  jmp       redraw_page
no_inc_width:

;----> Decrement Modulo? <----------------------------------------------------
  cmp       al,34h                  ; '4'?
  jNE       no_dec_modulo
  mov       ax,ram_modulo
  dec       ax
  JL        wait_keyboard
  mov       ram_modulo,ax
  dec       ram_total
  jmp       redraw_page
no_dec_modulo:

;----> Increment Modulo? <----------------------------------------------------
  cmp       al,35h                  ; '5'?
  jNE       no_inc_modulo
  inc       word ptr ram_modulo
  inc       ram_total
  jmp       redraw_page
no_inc_modulo:

;----> Reinit Values? <-------------------------------------------------------
  cmp       al,30h                  ; '0'?
  jE        init_values

;----> Put Greys1? <----------------------------------------------------------
  cmp       ax,3b00h                ; 'F1'?
  jNE       no_put_greys1
  mov       ax,cs
  mov       es,ax
  mov       di,offset cs:palette
  mov       bl,0
@@greys1_loop:
  mov       al,bl
  shr       al,2
  stosb
  stosb
  stosb
  inc       bl
  jNZ       @@greys1_loop
  call      put_palette
  mov       ax,0a000h
  mov       es,ax
no_put_greys1:

;----> Put Greys2? <----------------------------------------------------------
  cmp       ax,3c00h                ; 'F1'?
  jNE       no_put_greys2
  mov       ax,cs
  mov       es,ax
  mov       di,offset cs:palette
  mov       al,0
@@greys2_loop:
  stosb
  stosb
  stosb
  inc       al
  jNZ       @@greys2_loop
  call      put_palette
  mov       ax,0a000h
  mov       es,ax
no_put_greys2:

;----> Put Colors? <----------------------------------------------------------
  cmp       ax,3d00h                ; 'F3'?
  jNE       no_put_colors
  mov       ax,cs
  mov       es,ax
  mov       di,offset cs:palette
  mov       eax,ram_pointer
  mov       ebx,eax
  shr       ebx,4
  mov       ds,bx
  and       ax,0fh
  mov       si,ax
  mov       cx,3*256
  rep movsb
  call      put_palette
  mov       ax,0a000h
  mov       es,ax
no_put_colors:

;----> Save Image? <----------------------------------------------------------
  cmp       al,73h                  ; 's'?
  jNE       no_save_img
  mov       word ptr scr_pointer,0  ; Redraw image at 0a000:0000.
  mov       bx,199
@@redraw_for_saving:
  call      copy_line
  dec       bx
  jGE       @@redraw_for_saving
  mov       ah,3ch                  ; Create file.
  xor       cx,cx                   ;   normal file.
  mov       di,cs                   ;   ds:dx points on filename.
  mov       ds,di
  mov       dx,offset cs:filename
  int       21h
  jC        proggy_end              ; Quit if an error occured.
  mov       bx,ax                   ; bx=file handle.
  mov       di,dx
  inc       byte ptr ds:[di]        ; Change name for next file.
  mov       ah,40h                  ; Save sci header and palette.
  mov       cx,778                  ;   header is 778 bytes long.
  mov       dx,offset cs:sci_header ;   save cs:sci_header.
  int       21h
  jC        proggy_end
  mov       ah,40h                  ; Save image.
  mov       cx,320*scr_height       ;   size to save.
  mov       di,es                   ;   save from 0a000:0000.
  mov       ds,di
  xor       dx,dx
  int       21h
  jC        proggy_end              ; Quit if an error occured.
  mov       ah,3eh                  ; Close file.
  int       21h
  jC        proggy_end              ; Quit if an error occured.
  mov       bx,dx
no_save_img:

;----> Escape? <--------------------------------------------------------------
  cmp       al,1bh                  ; Escape?
  jNE       wait_keyboard           ; Nooooooo!
proggy_end:
  mov       ax,3h                   ; Back to text mode.
  int       10h
  mov       ah,4ch                  ; Terminate proggy function.
  int       21h


; ****************************************************************************
; ****************************************************************************
; ********************************  ROUTINES  ********************************
; ****************************************************************************
; ****************************************************************************


;---- Copy line --------------------------------------------------------------
; bx - line nb. (0-199)
copy_line   PROC NEAR
  pusha
  push ds

  mov       di,scr_pointer          ; Make di point on dest.
  mov       ax,320
  mul       bx                      ; dx:ax=320*line_nb.
  add       di,ax                   ; di=scr_offset+320*line_nb.

  mov       ax,ram_total            ; Make ds:si point on src.
  mul       bx                      ; dx:ax=(width+modulo)*line_nb.
  shl       edx,16
  mov       dx,ax                   ; edx=(width+modulo)*line_nb.
  add       edx,ram_pointer
  mov       ecx,edx
  shr       ecx,4                   ; cx=(width+modulo)*line_nb/16.
  mov       ds,cx
  and       dx,0fh                  ; dx=(width+modulo)*line_nb mod16.
  mov       si,dx

  mov       bx,ram_width
  mov       cx,bx
  rep movsb
  xor       al,al
  mov       cx,320                  ; cx=320-width.
  sub       cx,bx
  rep stosb

  pop ds
  popa
  ret
copy_line   ENDP

;---- Put Palette ------------------------------------------------------------
put_palette             PROC NEAR
  mov       ax,cs
  mov       ds,ax
  mov       si,offset cs:palette
  mov       bl,0                    ; bl=color counter.
  mov       dx,03c8h                ; Palette write port.
@@put_palette_one:
  mov       al,bl                   ; Color number in al.
  out       dx,al                   ; Choose color to change.
  inc       dx
  lodsb                             ; al=red compound.
  out       dx,al
  lodsb                             ; al=green compound.
  out       dx,al
  lodsb                             ; al=blue compound.
  out       dx,al
  dec       dx
  inc       bl                      ; One color done.
  jne       @@put_palette_one       ; If bl=256=0 then we have changed all colors.
  ret
put_palette             ENDP

;---- Datas ------------------------------------------------------------------

scr_pointer dw 0000h
ram_pointer dd 0h
ram_total   dw 0
ram_width   dw 0
ram_modulo  dw 0
filename    db          '0.sci',0

sci_header  label byte
  db        'RIX3'
  dw        320,200,175
palette     label byte
  incbin 'default.pal',3*256

THE_CODE    ENDS

; ***************************************************************************
; ***************************************************************************
; *******************************  STACK  ***********************************
; ***************************************************************************
; ***************************************************************************
THE_STACK               SEGMENT STACK
  db        '                                        '
  db        '          - JACK! THE RIPPER -          '
  db        '     By Alain BROBECKER. (aka baah)     '
  db        '        v1.00 - 24 november 1995        '
  db        '                                        '
THE_STACK               ENDS

                        END proggy_start
