; ****************************************************************************
; ******                                                                ******
; ******                      - JACK! THE RIPPER -                      ******
; ******                 By Alain BROBECKER. (aka baah)                 ******
; ******                   v1.1 - 24,25 november 1995                   ******
; ******                                                                ******
; ****************************************************************************

            include rincbin.inc
            scr_height=200

            .386


; ****************************************************************************
; *******************************              *******************************
; *******************************     CODE     *******************************
; *******************************              *******************************
; ****************************************************************************
THE_CODE    SEGMENT USE16
            Assume CS:THE_CODE,SS:THE_STACK

;====> FILE MANAGING <========================================================

memory_release:
  mov       bx,ss                   ; Calculate the size between the end of
  mov       ax,es                   ; stack and the beginning of psp.
  sub       bx,ax
  mov       ax,sp                   ; Calculate end of stack.
  shr       ax,4
  inc       ax                      ; In case stack is not a multiple of 16.
  add       bx,ax                   ; Here bx=size of proggy/16.
  mov       ah,4ah                  ; Allocated memory modify function.
  int       21h
  jC        proggy_end

  mov       bl,es:[80h]             ; bl=nb of bytes in command line.
  cmp       bl,0                    ; If nb of bytes is 0, no command line.
  jE        ripper_start
;----> Load file <----
  xor       bh,bh                   ; Else, we assume it is a filename.
  mov       byte ptr es:[bx+81h],0  ; Put 0 at the end of it.
  mov       ax,03d00h               ; Open file function. (read only)
  mov       di,es                   ; Make ds:dx point on filename.
  mov       ds,di
  mov       dx,82h                  ; (first char is space)
  int       21h
  jC        proggy_end
  mov       bx,ax                   ; bx=handle.
  mov       ax,4202h                ; Get size of file.
  xor       cx,cx
  xor       dx,dx
  int       21h                     ; Here dx:ax=file size.
  jC        proggy_end
  shl       edx,16
  mov       dx,ax                   ; edx=file size.
  cmp       edx,400000
  jLE       @@filesize_ok
  mov       edx,400000
@@filesize_ok:
  mov       edi,edx                 ; edi=file size.
  mov       ax,4200h                ; Restore file pointer.
  xor       cx,cx
  xor       dx,dx
  int       21h
  jC        proggy_end
  push      bx                      ; Save file handle.
  mov       edx,edi
  shr       edx,4                   ; dx=nb of segments to allocate-1.
  inc       dx
  mov       ah,48h                  ; MemAlloc function.
  mov       bx,dx                   ; Nb of segments to allocate.
  int       21h
  jC        proggy_end
  mov       si,ax                   ; Buffer segment.
  pop       bx                      ; bx=handle.
  xor       dx,dx
  mov       ecx,32768
@@read_32768:
  sub       edi,ecx                 ; Size-=32768.
  jL        @@read_last
  mov       ah,3fh                  ; Read function.
  mov       ds,si                   ; ds=segment buffer.
  int       21h
  jC        proggy_end
  add       si,32768/16             ; buffer adress+=32768.
  jmp       @@read_32768
@@read_last:
  add       cx,di                   ; cx=size.
  mov       ds,si                   ; ds=segment buffer.
  mov       ah,3fh                  ; Read function.
  int       21h
  jC        proggy_end

;====> THE RIPPER <===========================================================
ripper_start:
  mov       ax,13h                  ; Switch to 320*200 linear mode.
  int       10h
  mov       ax,0a000h               ; Put screen base in es.
  mov       es,ax

  mov       ax,0305h                ; Set keyboard repetition speed.
  mov       bx,0001h
  int       16h


redrawing:
  call      copy_screen

wait_keyboard:
  mov       bx,video_base           ; Set screen to videoram adress.
  shr       bx,2
  mov       dx,03d4h                ; CRT Controller.
  mov       al,0ch                  ; Start adress high subregister.
  mov       ah,bh
  out       dx,ax
  inc       al                      ; Start adress low subregister.
  mov       ah,bl
  out       dx,ax

  mov       ah,01h                  ; Wait until a char is in keyboard buffer.
  int       16h
  jZ        wait_keyboard
  xor       ah,ah                   ; Read char.
  int       16h

  cmp       al,30h
  jE        restore_base

  cmp       al,31h
  jE        dec_width
  cmp       al,32h
  jE        inc_width
  cmp       al,33h
  jE        restore_width

  cmp       al,34h
  jE        dec_modulo
  cmp       al,35h
  jE        inc_modulo
  cmp       al,36h
  jE        restore_modulo

  cmp       ax,3b00h
  jE        greys1
  cmp       ax,3c00h
  jE        greys2
  cmp       ax,3d00h

  cmp       al,73h
  jE        save_file

  cmp       ax,5000h
  jE        scroll_up
  cmp       ax,4800h
  jE        scroll_down
  cmp       ax,5100h
  jE        fast_scroll_up
  cmp       ax,4900h
  jE        fast_scroll_down

  cmp       ax,7400h
  jE        scroll_left
  cmp       ax,7300h
  jE        scroll_right
  cmp       ax,4d00h
  jE        fast_scroll_left
  cmp       ax,4b00h
  jE        fast_scroll_right

  cmp       al,1bh                  ; Escape key pressed?
  jNE       wait_keyboard

proggy_end:
  mov       ax,3h                   ; Back to text mode.
  int       10h
  mov       ah,4ch                  ; Terminate proggy function.
  int       21h




;-----------------------------------------------------------------------------
restore_base:
  mov       ram_base,0              ; Point on adress 0.
  jmp       redrawing

;-----------------------------------------------------------------------------
dec_width:
  mov       ax,img_width            ; Load previous width.
  sub       ax,8                    ; img_width-=8.
  jLE       wait_keyboard           ; No change if result is negative.
  mov       img_width,ax            ; Save new width.
  shl       ax,5                    ; ah=img_width/3.
  mov       dx,03d4h                ; Crt Controller Index.
  mov       al,13h                  ; Offset subregister.
  out       dx,ax                   ; Change virtual width.
  jmp       redrawing

inc_width:
  mov       ax,img_width            ; Load previous width.
  add       ax,8                    ; img_width+=8.
  cmp       ax,320
  jG        wait_keyboard           ; No change if img_width>320.
  mov       img_width,ax            ; Save new width.
  shl       ax,5                    ; ah=img_width/3.
  mov       dx,03d4h                ; Crt Controller Index.
  mov       al,13h                  ; Offset subregister.
  out       dx,ax                   ; Change virtual width.
  jmp       redrawing

restore_width:
  mov       img_width,320           ; Restore width.
  mov       dx,03d4h                ; Crt Controller Index.
  mov       ax,02813h               ; Offset subregister.
  out       dx,ax                   ; Change virtual width.
  jmp       redrawing

;-----------------------------------------------------------------------------
dec_modulo:
  mov       ax,img_modulo           ; Load previous modulo.
  dec       ax                      ; img_modulo-=1.
  jL        wait_keyboard           ; No change if result is negative.
  mov       img_modulo,ax           ; Save new modulo.
  jmp       redrawing

inc_modulo:
  inc       img_modulo              ; img_modulo+=1.
  jmp       redrawing

restore_modulo:
  mov       img_modulo,0            ; img_modulo=0.
  jmp       redrawing

;-----------------------------------------------------------------------------
greys1:
  mov       ax,cs                   ; Make cs:di point on sci_palette.
  mov       es,ax
  mov       di,offset cs:sci_palette
  mov       al,0                    ; al=color counter.
@@greys1_loop:
  mov       cx,3*4                  ; 4 colors at same grey value.
  rep stosb
  inc       al                      ; 4 colors set.
  cmp       al,64
  jNZ       @@greys1_loop
  call      put_palette
  mov       ax,0a000h               ; Restore es.
  mov       es,ax
  jmp       wait_keyboard



greys2:
  mov       ax,cs                   ; Make cs:di point on sci_palette.
  mov       es,ax
  mov       di,offset cs:sci_palette
  mov       al,0                    ; al=color counter.
@@greys2_loop:
  stosb                             ; Save grey value.
  stosb
  stosb
  inc       al                      ; One grey copied.
  jNZ       @@greys2_loop           ; Continue until al=256=0.
  call      put_palette
  mov       ax,0a000h               ; Restore es.
  mov       es,ax
  jmp       wait_keyboard


;-----------------------------------------------------------------------------
save_file:
  call      copy_screen             ; Redraw image at top of videoram.
  mov       ah,3ch                  ; Create file.
  xor       cx,cx                   ;   normal file.
  mov       di,cs                   ;   ds:dx points on filename.
  mov       ds,di
  mov       dx,offset cs:filename
  int       21h
  jC        proggy_end              ; Quit if an error occured.
  mov       bx,ax                   ; bx=file handle.
  mov       di,dx
  inc       byte ptr ds:[di]        ; Change name for next file.
  mov       ah,40h                  ; Save sci header and palette.
  mov       cx,778                  ;   header is 778 bytes long.
  mov       dx,offset cs:sci_header ;   save cs:sci_header.
  int       21h
  jC        proggy_end
  mov       ax,img_width            ; Save raw image.
  mov       cx,scr_height
  mul       cx                      ;   ax=width*200=size of image.
  mov       cx,ax                   ;   size in cx.
  mov       ah,40h                  ;   function nb in ah.
  mov       di,es                   ;   save from 0a000:0000.
  mov       ds,di
  xor       dx,dx
  int       21h
  jC        proggy_end              ; Quit if an error occured.
  mov       ah,3eh                  ; Close file.
  int       21h
  jC        proggy_end              ; Quit if an error occured.
  jmp       wait_keyboard

;-----------------------------------------------------------------------------
scroll_up:
  xor       eax,eax
  mov       ax,img_width
  add       video_base,ax           ; video_base+=width.
  add       ax,img_modulo           ; eax=width+modulo.
  add       ram_base,eax            ; ram_base+=(width+modulo).
  mov       bx,scr_height-1
  call      copy_line
  jmp       wait_keyboard

scroll_down:
  xor       eax,eax
  mov       ax,img_width
  sub       video_base,ax           ; video_base-=width.
  add       ax,img_modulo           ; eax=width+modulo.
  sub       ram_base,eax            ; ram_base-=(width+modulo).
  jGE       @@scroll_down_ok
  mov       ram_base,0              ; In case ram_base<0, set it to 0
  jmp       redrawing               ; and redraw the screen.
@@scroll_down_ok:
  xor       bx,bx
  call      copy_line
  jmp       wait_keyboard

fast_scroll_up:
  mov       ax,img_width
  add       ax,img_modulo           ; ax=(width+modulo).
  mov       bx,scr_height
  mul       bx                      ; dx:ax=(width+modulo)*height.
  shl       edx,16
  mov       dx,ax                   ; edx=(width+modulo)*height.
  add       ram_base,edx            ; ram_base+=(width+modulo)*height.
  jmp       redrawing


fast_scroll_down:
  mov       ax,img_width
  add       ax,img_modulo           ; ax=(width+modulo).
  mov       bx,scr_height
  mul       bx                      ; dx:ax=(width+modulo)*height.
  shl       edx,16
  mov       dx,ax                   ; edx=(width+modulo)*height.
  sub       ram_base,edx            ; ram_base-=(width+modulo)*height.
  jGE       redrawing
  mov       ram_base,0
  jmp       redrawing


;-----------------------------------------------------------------------------
scroll_left:
  inc       ram_base                ; ram_base+=1.
  jmp       redrawing

scroll_right:
  dec       ram_base                ; ram_base-=1.
  jGE       redrawing
  mov       ram_base,0              ; If ram_base was<0, set it to 0.
  jmp       redrawing


;-----------------------------------------------------------------------------
fast_scroll_left:
  mov       edx,ram_base
  mov       ecx,edx
  add       ecx,4                   ; ecx=ram_base+4.
  mov       ram_base,ecx
  mov       di,video_base
  mov       cx,di
  add       cx,4                    ; cx=video_base+4.
  mov       video_base,cx
  xor       eax,eax
  mov       ax,img_width
  mov       ebx,eax
  add       bx,img_modulo           ; ebx=width+modulo.
  add       di,ax                   ; di=new_video_base+img_width-4.
  add       edx,eax                 ; edx=new_ram_base+img_width-4.
  sub       ax,4                    ; ax=img_width-4.
  mov       cl,scr_height           ; Nb of lines.
@@fast_left_loop:
  mov       esi,edx                 ; Make ds:si=edx.
  shr       esi,4                   ;   si=(width+modulo)*line_nb/16.
  mov       ds,si                   ;   ds=(width+modulo)*line_nb/16.
  mov       si,dx
  and       si,0fh                  ;   si=(width+modulo)*line_nb mod16.
  movsb                             ; Copy 4 pixels.
  movsb
  movsb
  movsb
  add       di,ax                   ; dest+=width-4.
  add       edx,ebx                 ; src+=width+modulo.
  dec       cl                      ; One line processed.
  jNE       @@fast_left_loop
  jmp       wait_keyboard


fast_scroll_right:
  mov       edx,ram_base
  sub       edx,4                   ; edx=ram_base-4.
  jGE       @@fast_right_ok
  mov       ram_base,0
  jmp       redrawing
@@fast_right_ok:
  mov       ram_base,edx
  mov       di,video_base
  sub       di,4                    ; di=video_base-4.
  mov       video_base,di
  xor       eax,eax
  mov       ax,img_width
  mov       ebx,eax
  add       bx,img_modulo           ; ebx=width+modulo.
  sub       ax,4                    ; ax=img_width-4.
  mov       cl,scr_height           ; Nb of lines.
@@fast_right_loop:
  mov       esi,edx                 ; Make ds:si=edx.
  shr       esi,4                   ;   si=(width+modulo)*line_nb/16.
  mov       ds,si                   ;   ds=(width+modulo)*line_nb/16.
  mov       si,dx
  and       si,0fh                  ;   si=(width+modulo)*line_nb mod16.
  movsb                             ; Copy 4 pixels.
  movsb
  movsb
  movsb
  add       di,ax                   ; dest+=width-4.
  add       edx,ebx                 ; src+=width+modulo.
  dec       cl                      ; One line processed.
  jNE       @@fast_right_loop
  jmp       wait_keyboard


;==== Copy Screen ============================================================
copy_screen             PROC NEAR
  xor       di,di                   ; Set video base to 0.
  mov       video_base,di
  mov       ebp,ram_base            ; Put ram base in ds:si.
  mov       ax,scr_height           ; ax=nb of lines to copy.
  xor       ebx,ebx
  mov       bx,img_width            ; bx=nb of bytes to copy per line.
  mov       edx,ebx
  add       dx,img_modulo           ; edx=width+modulo.
@@copy_screen_loop:
  mov       si,bp
  and       si,0fh                  ; si=ram_base mod16.
  mov       ecx,ebp
  shr       ecx,4
  mov       ds,cx                   ; ds=int(ram_base/16).
  mov       cx,bx                   ; Nb of bytes to copy.
  rep movsb
  add       ebp,edx                 ; ram+=width+modulo.
  dec       ax                      ; One line copied.
  jNE       @@copy_screen_loop
  ret
copy_screen             ENDP
;=============================================================================


;==== Copy Line ==============================================================
; bx - line nb. (0-scr_height-1)
copy_line               PROC NEAR
  mov       di,video_base           ; Make es:di point on dest.
  mov       ax,img_width            ;   width bytes per line.
  mul       bx                      ;   dx:ax=width*line_nb.
  add       di,ax                   ;   di=scr_offset+width*line_nb.
  mov       ax,img_width            ; Make ds:si point on src.
  add       ax,img_modulo
  mul       bx                      ;   dx:ax=(width+modulo)*line_nb.
  shl       edx,16
  mov       dx,ax                   ;   edx=(width+modulo)*line_nb.
  add       edx,ram_base
  mov       si,dx
  and       si,0fh                  ;   si=(width+modulo)*line_nb mod16.
  shr       edx,4                   ;   dx=(width+modulo)*line_nb/16.
  mov       ds,dx
  mov       cx,img_width            ; Copy img_width bytes.
  rep movsb
  ret
copy_line               ENDP
;=============================================================================



;==== Put Palette ============================================================
put_palette             PROC NEAR
  xor       cl,cl                   ; cl=color counter.
  mov       si,offset cs:sci_palette ; cs:[bx] points on palette.
  mov       ax,cs
  mov       ds,ax
  mov       dx,03c8h                ; Palette write port.
@@put_palette_one:
  mov       al,cl                   ; al=color to change.
  out       dx,al
  inc       dx
  lodsb                             ; Load red compound.
  out       dx,al
  lodsb                             ; Load red compound.
  out       dx,al
  lodsb                             ; Load red compound.
  out       dx,al
  dec       dx
  inc       cl                      ; One color changed.
  jne       @@put_palette_one       ; Change until cl=256=0.
  ret
put_palette             ENDP
;=============================================================================

;---- Datas ------------------------------------------------------------------

video_base  dw 0
ram_base    dd 0

img_modulo  dw 0

filename    db '0.sci',0            ; Name of the file.


sci_header  db 'RIX3'               ; The following is the hader for files.
img_width   dw 320
img_height  dw scr_height
sci_zarbi   dw 175
sci_palette label byte
  incbin 'default.pal',3*256


THE_CODE    ENDS




; ***************************************************************************
; ***************************************************************************
; *******************************  STACK  ***********************************
; ***************************************************************************
; ***************************************************************************
THE_STACK               SEGMENT STACK
  db        '                                        '
  db        '          - JACK! THE RIPPER -          '
  db        '     By Alain BROBECKER. (aka baah)     '
  db        '       v1.1 - 24,25 november 1995       '
  db        '                                        '
THE_STACK               ENDS

                        END memory_release
